@@ -1,7 +1,7 @@
 #' The YAML metadata of the current R Markdown document
-#' 
+#'
 #' The object \code{metadata} stores the YAML metadata of the current R Markdown
-#' document as a list, which you may use in the R code chunks, e.g. 
+#' document as a list, which you may use in the R code chunks, e.g.
 #' \code{rmarkdown::metadata$title} (the title of the document),
 #' \code{rmarkdown::metadata$author}, and \code{rmarkdown::metadata$foo} (if you
 #' have a YAML field named \code{foo}), etc.
@@ -20,7 +20,9 @@ render <- function(input,
                    runtime = c("auto", "static", "shiny"),
                    clean = TRUE,
                    params = NULL,
+                   knit_meta = NULL,
                    envir = parent.frame(),
+                   run_pandoc = TRUE,
                    quiet = FALSE,
                    encoding = getOption("encoding")) {
 
@@ -49,7 +51,9 @@ render <- function(input,
                        runtime = runtime,
                        clean = clean,
                        params = params,
+                       knit_meta = knit_meta,
                        envir = envir,
+                       run_pandoc = run_pandoc,
                        quiet = quiet,
                        encoding = encoding)
       outputs <- c(outputs, output)
@@ -76,26 +80,26 @@ render <- function(input,
   if (!is.null(intermediates_dir)) {
     if (!dir_exists(intermediates_dir))
       dir.create(intermediates_dir, recursive = TRUE)
-    intermediates_dir <- normalizePath(intermediates_dir, winslash = "/")
-  } 
+    intermediates_dir <- normalize_path(intermediates_dir)
+  }
   intermediates_loc <- function(file) {
     if (is.null(intermediates_dir))
       file
     else
       file.path(intermediates_dir, file)
   }
-  
-  # resolve output directory before we change the working directory in 
+
+  # resolve output directory before we change the working directory in
   # preparation for rendering the document
   if (!is.null(output_dir)) {
     if (!dir_exists(output_dir))
       dir.create(output_dir, recursive = TRUE)
-    output_dir <- normalizePath(output_dir, winslash = "/")
+    output_dir <- normalize_path(output_dir)
   }
-  
+
   # remember the name of the original input document (we overwrite 'input' once
   # we've knitted)
-  original_input <- normalizePath(input, winslash = "/")
+  original_input <- normalize_path(input)
 
   # if the input file has shell characters in its name then make a copy that
   # doesn't have shell characters
@@ -103,25 +107,25 @@ render <- function(input,
     # form the name of the file w/o shell characters
     input_no_shell_chars <- intermediates_loc(
         file_name_without_shell_chars(basename(input)))
-    
+
     if (file.exists(input_no_shell_chars)) {
       stop("The name of the input file cannot contain the special shell ",
-           "characters: ", .shell_chars_regex, " (attempted to copy to a ", 
+           "characters: ", .shell_chars_regex, " (attempted to copy to a ",
            "version without those characters '", input_no_shell_chars, "' ",
            "however that file already exists)", call. = FALSE)
     }
     file.copy(input, input_no_shell_chars, overwrite = TRUE)
     intermediates <- c(intermediates, input_no_shell_chars)
     input <- input_no_shell_chars
-    
+
     # if an intermediates directory wasn't explicit before, make it explicit now
     if (is.null(intermediates_dir)) {
-      intermediates_dir <- 
-        dirname(normalizePath(input_no_shell_chars, winslash = "/"))
+      intermediates_dir <-
+        dirname(normalize_path(input_no_shell_chars))
       # never use the original input directory as the intermediate directory,
       # otherwise external resources discovered will be deleted as intermediate
       # files later (because they are copied to the "intermediate" dir)
-      if (intermediates_dir == normalizePath(dirname(original_input), winslash = "/"))
+      if (same_path(intermediates_dir, dirname(original_input)))
         intermediates_dir <- NULL
     }
   }
@@ -205,17 +209,16 @@ render <- function(input,
   files_dir <-file.path(output_dir, knitr_files_dir(basename(output_file)))
   files_dir <- pandoc_path_arg(files_dir)
 
-  # default to no cache_dir knit_meta (some may be generated by the knit)
+  # default to no cache_dir (may be generated by the knit)
   cache_dir <- NULL
-  knit_meta <- NULL
 
   # call any intermediate files generator, if we have an intermediates directory
   # (do this before knitting in case the knit requires intermediates)
   if (!is.null(intermediates_dir) &&
       !is.null(output_format$intermediates_generator)) {
-    intermediates <- c(intermediates, 
-                       output_format$intermediates_generator(original_input, 
-                                                             encoding, 
+    intermediates <- c(intermediates,
+                       output_format$intermediates_generator(original_input,
+                                                             encoding,
                                                              intermediates_dir))
   }
 
@@ -293,7 +296,7 @@ render <- function(input,
     if (!is.null(yaml_front_matter$params)) {
 
       params <- knit_params_get(input_lines, params)
-     
+
       # make the params available in the knit environment
       if (!exists("params", envir = envir, inherits = FALSE)) {
         assign("params", params, envir = envir)
@@ -307,7 +310,7 @@ render <- function(input,
              "so can't be overwritten by render params", call. = FALSE)
       }
     }
-    
+
     # make the yaml_front_matter available as 'metadata' within the
     # knit environment (unless it is already defined there in which case
     # we emit a warning)
@@ -350,7 +353,7 @@ render <- function(input,
         if (!isTRUE(yaml_front_matter$always_allow_html)) {
           stop("Functions that produce HTML output found in document targeting ",
                pandoc_to, " output.\nPlease change the output type ",
-               "of this document to HTML. Alternatively, you can allow\n", 
+               "of this document to HTML. Alternatively, you can allow\n",
                "HTML output in non-HTML formats by adding this option to the YAML front",
                "-matter of\nyour rmarkdown file:\n\n",
                "  always_allow_html: yes\n\n",
@@ -375,84 +378,91 @@ render <- function(input,
   input_text <- read_lines_utf8(input, encoding)
   writeLines(input_text, utf8_input, useBytes = TRUE)
 
-  perf_timer_start("pre-processor")
+  if (run_pandoc) {
 
-  # call any pre_processor
-  if (!is.null(output_format$pre_processor)) {
-    extra_args <- output_format$pre_processor(yaml_front_matter,
-                                              utf8_input,
-                                              runtime,
-                                              knit_meta,
-                                              files_dir,
-                                              output_dir)
-    output_format$pandoc$args <- c(output_format$pandoc$args, extra_args)
-  }
-  
-  perf_timer_stop("pre-processor")
+    perf_timer_start("pre-processor")
 
-  # if we are running citeproc then explicitly forward the bibliography
-  # on the command line (works around pandoc-citeproc issue whereby yaml
-  # strings that begin with numbers are interpreted as numbers)
-  if (!is.null(yaml_front_matter$bibliography)) {
-    output_format$pandoc$args <- c(output_format$pandoc$args,
-      rbind("--bibliography", pandoc_path_arg(yaml_front_matter$bibliography)))
-  }
-  
-  perf_timer_start("pandoc")
-
-  convert <- function(output, citeproc = FALSE) {
-    pandoc_convert(
-      utf8_input, pandoc_to, output_format$pandoc$from, output, citeproc,
-      output_format$pandoc$args, !quiet
-    )
-  }
-  texfile <- file_with_ext(output_file, "tex")
-  # compile Rmd to tex when we need to generate bibliography with natbib/biblatex
-  if (grepl('[.](pdf|tex)$', output_file) &&
-      any(c('--natbib', '--biblatex') %in% output_format$pandoc$args)) {
-    convert(texfile)
-    # manually compile tex if PDF output is expected
-    if (grepl('[.]pdf$', output_file)) {
-      latexmk(texfile, output_format$pandoc$latex_engine)
-      file.rename(file_with_ext(texfile, "pdf"), output_file)
-    }
-    # clean up the tex file if necessary
-    if ((texfile != output_file) && !output_format$pandoc$keep_tex)
-      on.exit(unlink(texfile), add = TRUE)
-  } else {
-    # run the main conversion if the output file is not .tex
-    convert(output_file, run_citeproc)
-    # run conversion again to .tex if we want to keep the tex source
-    if (output_format$pandoc$keep_tex) convert(texfile, run_citeproc)
-  }
-  
-  # pandoc writes the output alongside the input, so if we rendered from an 
-  # intermediate directory, move the output file
-  if (!is.null(intermediates_dir)) {
-    intermediate_output <- file.path(intermediates_dir, basename(output_file))
-    if (file.exists(intermediate_output)) {
-      file.rename(intermediate_output, output_file)
-    }
-  }
-
-  perf_timer_stop("pandoc")
-
-  perf_timer_start("post-processor")
-
-  # if there is a post-processor then call it
-  if (!is.null(output_format$post_processor))
-    output_file <- output_format$post_processor(yaml_front_matter,
+    # call any pre_processor
+    if (!is.null(output_format$pre_processor)) {
+      extra_args <- output_format$pre_processor(yaml_front_matter,
                                                 utf8_input,
-                                                output_file,
-                                                clean,
-                                                !quiet)
+                                                runtime,
+                                                knit_meta,
+                                                files_dir,
+                                                output_dir)
+      output_format$pandoc$args <- c(output_format$pandoc$args, extra_args)
+    }
+
+    perf_timer_stop("pre-processor")
+
+    need_bibtex <- grepl('[.](pdf|tex)$', output_file) &&
+      any(c('--natbib', '--biblatex') %in% output_format$pandoc$args)
+    # if we are running citeproc then explicitly forward the bibliography
+    # on the command line (works around pandoc-citeproc issue whereby yaml
+    # strings that begin with numbers are interpreted as numbers)
+    if (!is.null(bibliography <- yaml_front_matter$bibliography)) {
+      # remove the .bib extension since it does not work with MikTeX's BibTeX
+      if (need_bibtex && is_windows()) bibliography <- sub('[.]bib$', '', bibliography)
+      output_format$pandoc$args <- c(output_format$pandoc$args,
+                                     rbind("--bibliography", pandoc_path_arg(bibliography)))
+    }
+
+    perf_timer_start("pandoc")
+
+    convert <- function(output, citeproc = FALSE) {
+      pandoc_convert(
+        utf8_input, pandoc_to, output_format$pandoc$from, output, citeproc,
+        output_format$pandoc$args, !quiet
+      )
+    }
+    texfile <- file_with_ext(output_file, "tex")
+    # compile Rmd to tex when we need to generate bibliography with natbib/biblatex
+    if (need_bibtex) {
+      convert(texfile)
+      # manually compile tex if PDF output is expected
+      if (grepl('[.]pdf$', output_file)) {
+        latexmk(texfile, output_format$pandoc$latex_engine)
+        file.rename(file_with_ext(texfile, "pdf"), output_file)
+      }
+      # clean up the tex file if necessary
+      if ((texfile != output_file) && !output_format$pandoc$keep_tex)
+        on.exit(unlink(texfile), add = TRUE)
+    } else {
+      # run the main conversion if the output file is not .tex
+      convert(output_file, run_citeproc)
+      # run conversion again to .tex if we want to keep the tex source
+      if (output_format$pandoc$keep_tex) convert(texfile, run_citeproc)
+    }
+
+    # pandoc writes the output alongside the input, so if we rendered from an
+    # intermediate directory, move the output file
+    if (!is.null(intermediates_dir)) {
+      intermediate_output <- file.path(intermediates_dir, basename(output_file))
+      if (file.exists(intermediate_output)) {
+        file.rename(intermediate_output, output_file)
+      }
+    }
+
+    perf_timer_stop("pandoc")
+
+    perf_timer_start("post-processor")
+
+    # if there is a post-processor then call it
+    if (!is.null(output_format$post_processor))
+      output_file <- output_format$post_processor(yaml_front_matter,
+                                                  utf8_input,
+                                                  output_file,
+                                                  clean,
+                                                  !quiet)
+
+    if (!quiet) {
+      message("\nOutput created: ", relative_to(oldwd, output_file))
+    }
+
+    perf_timer_stop("post-processor")
 
-  if (!quiet) {
-    message("\nOutput created: ", relative_to(oldwd, output_file))
   }
 
-  perf_timer_stop("post-processor")
-
   perf_timer_stop("render")
 
   # write markdown output if requested
@@ -464,8 +474,14 @@ render <- function(input,
     writeLines(md, file_with_ext(output_file, "md"), useBytes = TRUE)
   }
 
-  # return the full path to the output file
-  invisible(tools::file_path_as_absolute(output_file))
+  if (run_pandoc) {
+    # return the full path to the output file
+    invisible(tools::file_path_as_absolute(output_file))
+  } else {
+    # did not run pandoc; returns the markdown output with attributes of the
+    # knitr meta data and intermediate files
+    structure(input, knit_meta = knit_meta, intermediates = intermediates)
+  }
 }
 
 
